import unittest
from xml.etree import ElementTree as ET

from slixmpp import Iq
from slixmpp.plugins import xep_0462
from slixmpp.plugins.xep_0004.stanza import Form, FormField
from slixmpp.plugins.xep_0030.stanza.items import DiscoItems
from slixmpp.test import SlixTest
from slixmpp.xmlstream import register_stanza_plugin


class TestPubsubTypeFiltering(SlixTest):
    def setUp(self):
        register_stanza_plugin(Iq, DiscoItems)
        register_stanza_plugin(Form, FormField, iterable=True)
        xep_0462.stanza.register_plugin()

    def test_setting(self):
        iq = Iq()
        iq["disco_items"]["filter"]["included_types"] = ["t1", "t2"]
        assert iq["disco_items"]["filter"]["included_types"] == ["t1", "t2"]
        self.check(
            iq,
            """
            <iq id='0'>
              <query xmlns='http://jabber.org/protocol/disco#items'>
                <filter xmlns='urn:xmpp:pubsub-filter:0'>
                  <x xmlns='jabber:x:data' type='submit'>
                    <field var='FORM_TYPE' type='hidden'>
                      <value>urn:xmpp:pubsub-filter:0</value>
                    </field>
                    <field var='included-types'>
                      <value>t1</value>
                      <value>t2</value>
                    </field>
                  </x>
                </filter>
              </query>
            </iq>
            """,
        )

        iq["disco_items"]["filter"]["included_types"] = ["t2", "t3"]
        self.assertEqual(iq["disco_items"]["filter"]["included_types"], ["t2", "t3"])
        self.check(
            iq,
            """
            <iq id='0'>
              <query xmlns='http://jabber.org/protocol/disco#items'>
                <filter xmlns='urn:xmpp:pubsub-filter:0'>
                  <x xmlns='jabber:x:data' type='submit'>
                    <field var='FORM_TYPE' type='hidden'>
                      <value>urn:xmpp:pubsub-filter:0</value>
                    </field>
                    <field var='included-types'>
                      <value>t2</value>
                      <value>t3</value>
                    </field>
                  </x>
                </filter>
              </query>
            </iq>
            """,
        )

    def test_parsing(self):
        iq = Iq(
            xml=ET.fromstring("""
            <iq type='get'
              from='rosa@com.int/desktop'
              to='news.commons.social'
              id='disco1'>
              <query xmlns='http://jabber.org/protocol/disco#items'>
                <filter xmlns='urn:xmpp:pubsub-filter:0'>
                  <x xmlns='jabber:x:data' type='submit'>
                    <field var='FORM_TYPE' type='hidden'>
                      <value>urn:xmpp:pubsub-filter:0</value>
                    </field>
                    <field type='list-multi' var='included-types'>
                      <value>urn:xmpp:microblog:0</value>
                    </field>
                  </x>
                </filter>
              </query>
            </iq>
            """)
        )
        self.assertEqual(
            iq["disco_items"]["filter"]["included_types"], ["urn:xmpp:microblog:0"]
        )

    def test_parsing_empty(self):
        iq = Iq(
            xml=ET.fromstring("""
            <iq type='get'
              from='rosa@com.int/desktop'
              to='news.commons.social'
              id='disco1'>
              <query xmlns='http://jabber.org/protocol/disco#items'>
                <filter xmlns='urn:xmpp:pubsub-filter:0' />
              </query>
            </iq>
            """)
        )
        self.assertEqual(iq["disco_items"]["filter"]["included_types"], [])


suite = unittest.TestLoader().loadTestsFromTestCase(TestPubsubTypeFiltering)



import unittest
from slixmpp.exceptions import IqError
from slixmpp.test import SlixTest


class TestLiveCam(SlixTest):

    def setUp(self):
        self.stream_start(plugins=['xep_0494'])

    def test_request_clients(self):
        """Request a client list"""

        clients = []

        async def run():
            clients.extend(await self.xmpp['xep_0494'].get_clients())

        self.xmpp.wrap(run())
        self.wait_()

        self.send("""
          <iq type="get" id="1">
            <list xmlns="urn:xmpp:cam:0"/>
          </iq>
        """)

        self.recv("""
<iq type="result" id="1">
  <clients xmlns="urn:xmpp:cam:0">
    <client connected="true" id="zeiP41HLglIu" type="session">
      <first-seen>2023-04-06T14:26:08Z</first-seen>
      <last-seen>2023-04-06T14:37:25Z</last-seen>
      <auth>
        <password/>
      </auth>
      <permission status="unrestricted"/>
      <user-agent>
        <software>Gajim</software>
        <uri>https://gajim.org/</uri>
        <device>Juliet's laptop</device>
      </user-agent>
    </client>
    <client connected="false" id="HjEEr45_LQr" type="access">
      <first-seen>2023-03-27T15:16:09Z</first-seen>
      <last-seen>2023-03-27T15:37:24Z</last-seen>
      <auth>
          <grant/>
      </auth>
      <permission status="normal"/>
      <user-agent>
        <software>REST client</software>
      </user-agent>
    </client>
  </clients>
</iq>
        """)
        self.wait_()
        self.assertEqual(len(clients), 2)
        self.assertTrue(clients[0]['connected'])
        self.assertEqual(clients[0]['type'].value, 'session')
        self.assertTrue(clients[0]['auth']['password'])
        self.assertFalse(clients[0]['auth']['fast'])
        self.assertEqual(clients[0]['permission'].value, 'unrestricted')
        self.assertEqual(clients[0]['user_agent']['software'], 'Gajim')
        self.assertFalse(clients[1]['connected'])

    def test_revoke_client(self):
        """Try to revoke a client"""

        result = []

        async def run():
            await self.xmpp['xep_0494'].revoke('toto')
            result.append(True)

        self.xmpp.wrap(run())
        self.wait_()

        self.send("""
          <iq type="get" id="1">
            <revoke xmlns="urn:xmpp:cam:0" id="toto"/>
          </iq>
        """)
        self.recv("""
          <iq type="result" id="1"/>
        """)
        self.wait_()

        self.assertTrue(result)

    def test_revoke_client_fail(self):
        """Try to revoke a client and fail"""

        result = []

        async def run():
            try:
                await self.xmpp['xep_0494'].revoke('toto')
            except IqError:
                result.append(True)

        self.xmpp.wrap(run())
        self.wait_()

        self.send("""
          <iq type="get" id="1">
            <revoke xmlns="urn:xmpp:cam:0" id="toto"/>
          </iq>
        """)
        self.recv("""
          <iq type="error" id="1"/>
        """)
        self.wait_()

        self.assertTrue(result)


suite = unittest.TestLoader().loadTestsFromTestCase(TestLiveCam)



import os
import unittest

from slixmpp.componentxmpp import ComponentXMPP
from slixmpp import JID

IN_CI = os.getenv('CI') == 'woodpecker'


class SlixComponentIntegration(unittest.IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.component = ComponentXMPP(CI_JID, CI_SECRET, CI_SERVER, CI_PORT)
        self.component.register_plugin("xep_0199")
        await self.component.connect()
        await self.component.wait_until("session_start")

    async def asyncTearDown(self):
        self.component.abort()

    @unittest.skipIf(not IN_CI, "Not in woodpecker CI")
    async def test_ping(self):
        """Check that a component can connect and send a ping to the server"""
        jid = JID(f"whatever@{CI_JID}")
        iq_pong = await self.component.plugin["xep_0199"].send_ping(JID(CI_SERVER), ifrom=jid)
        self.assertEqual(iq_pong["type"], "result")
        self.assertEqual(iq_pong.get_to(), jid)


CI_JID = "slixmpp.prosody"
CI_SECRET = "secret"
CI_SERVER = "prosody"
CI_PORT = 5347

suite = unittest.TestLoader().loadTestsFromTestCase(SlixComponentIntegration)



# Slixmpp: The Slick XMPP Library
# Copyright (C) 2025 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import unittest
from datetime import datetime
from slixmpp.test import SlixTest
from slixmpp.plugins.xep_0494 import XEP_0494
import slixmpp.plugins.xep_0494.stanza as stanza


class TestCAM(SlixTest):

    def setUp(self):
        stanza.register_plugins()

    def test_stanzabuild(self):
        """Testing creating a stanza."""

        xmlstring = """
<iq>
  <clients xmlns="urn:xmpp:cam:0">
    <client connected="true" id="zeiP41HLglIu" type="session">
      <first-seen>2023-04-06T14:26:08Z</first-seen>
      <last-seen>2023-04-06T14:37:25Z</last-seen>
      <auth>
        <password/>
      </auth>
      <permission status="unrestricted"/>
      <user-agent>
        <software>Gajim</software>
        <uri>https://gajim.org/</uri>
        <device>Juliet's laptop</device>
      </user-agent>
    </client>
    <client connected="false" id="HjEEr45_LQr" type="access">
      <first-seen>2023-03-27T15:16:09Z</first-seen>
      <last-seen>2023-03-27T15:37:24Z</last-seen>
      <auth>
          <grant/>
      </auth>
      <permission status="normal"/>
      <user-agent>
        <software>REST client</software>
      </user-agent>
    </client>
  </clients>
</iq>
        """

        iq = self.Iq()
        client = iq['clients']['client']
        client['connected'] = True
        client['first_seen'] = '2023-04-06T14:26:08Z'
        client['last_seen'] = '2023-04-06T14:37:25Z'
        client['id'] = 'zeiP41HLglIu'
        client['type'] = 'session'
        client['auth']['password'] = True
        client['permission'] = 'unrestricted'
        client['user_agent']['software'] = 'Gajim'
        client['user_agent']['uri'] = 'https://gajim.org/'
        client['user_agent']['device'] = 'Juliet\'s laptop'
        client2 = stanza.Client()
        iq['clients'].append(client2)
        client2['connected'] = False
        client2['first_seen'] = datetime.fromisoformat('2023-03-27T15:16:09Z')
        client2['last_seen'] = '2023-03-27T15:37:24Z'
        client2['id'] = 'HjEEr45_LQr'
        client2['type'] = 'access'
        client2['auth'].enable('grant')
        client2['permission'] = 'normal'
        client2['user_agent']['software'] = 'REST client'

        self.check(iq, xmlstring, use_values=False)


suite = unittest.TestLoader().loadTestsFromTestCase(TestCAM)



import unittest

from slixmpp.plugins.xep_0511 import stanza
from slixmpp.test import SlixTest


class TestLinkMetadata(SlixTest):
    def setUp(self):
        stanza.register_plugin()

    def testSetLinkMetadata(self):
        msg = self.Message()
        msg["link_metadata"]["about"] = (
            "https://the.link.example.com/what-was-linked-to"
        )
        msg["link_metadata"]["title"] = "A cool title"
        # This does not pass, probably because of namespace shenanigans in SlixText.check()
        # self.check(
        #     msg,  # language=xml
        #     """
        #     <message xmlns="jabber:client">
        #       <rdf:Description xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        #                        rdf:about="https://the.link.example.com/what-was-linked-to">
        #         <title xmlns="https://ogp.me/ns#">A cool title</title>
        #       </rdf:Description>
        #     </message>
        #     """,
        #     use_values=False,
        # )
        assert msg["link_metadata"]["description"] is None
        assert msg["link_metadata"]["title"] == "A cool title"
        assert msg["link_metadata"]["about"] == "https://the.link.example.com/what-was-linked-to"

    def testGetLinkMetadata(self):
        # language=xml
        xml_str = """
            <message to="whoever@example.com">
              <body>I wanted to mention https://the.link.example.com/what-was-linked-to</body>
              <rdf:Description xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                               xmlns:og="https://ogp.me/ns#"
                               rdf:about="https://the.link.example.com/what-was-linked-to">
                <og:title>Page Title</og:title>
                <og:description>Page Description</og:description>
                <og:url>Canonical URL</og:url>
                <og:type>website</og:type>
                <og:site_name>Some Website</og:site_name>
                <og:image>data:image/jpeg,...</og:image>
              </rdf:Description>
            </message>
            """
        xml = self.parse_xml(xml_str)
        msg = self.Message(xml)
        assert msg["link_metadata"]["title"] == "Page Title"
        assert msg["link_metadata"]["description"] == "Page Description"
        assert msg["link_metadata"]["url"] == "Canonical URL"
        assert msg["link_metadata"]["site_name"] == "Some Website"
        assert msg["link_metadata"]["type"] == "website"
        assert msg["link_metadata"]["image"] == "data:image/jpeg,..."
        assert (
            msg["link_metadata"]["about"]
            == "https://the.link.example.com/what-was-linked-to"
        )

    def testGetLinkMetadatas(self):
        # language=xml
        xml_str = """
            <message to="whoever@example.com">
              <body>I wanted to mention https://the.link.example.com/what-was-linked-to</body>
              <rdf:Description xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                               xmlns:og="https://ogp.me/ns#"
                               rdf:about="https://the.link.example.com/what-was-linked-to 1">
                <og:title>Page Title 1</og:title>
                <og:description>Page Description 1</og:description>
                <og:url>Canonical URL 1</og:url>
                <og:type>website 1</og:type>
                <og:site_name>Some Website 1</og:site_name>
                <og:image>https://link.to.example.com/image.png 1</og:image>
              </rdf:Description>
              <rdf:Description xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                               xmlns:og="https://ogp.me/ns#"
                               rdf:about="https://the.link.example.com/what-was-linked-to 2">
                <og:title>Page Title 2</og:title>
                <og:description>Page Description 2</og:description>
                <og:url>Canonical URL 2</og:url>
                <og:image>https://link.to.example.com/image.png 2</og:image>
                <og:type>website 2</og:type>
                <og:site_name>Some Website 2</og:site_name>
              </rdf:Description>
            </message>
            """
        xml = self.parse_xml(xml_str)
        msg = self.Message(xml)
        metadatas = msg["link_metadatas"]
        assert len(metadatas) == 2
        for i, p in enumerate(metadatas, start=1):
            assert p["title"] == f"Page Title {i}"
            assert p["description"] == f"Page Description {i}"
            assert p["url"] == f"Canonical URL {i}"
            assert p["image"] == f"https://link.to.example.com/image.png {i}"
            assert p["site_name"] == f"Some Website {i}"
            assert p["type"] == f"website {i}"
            assert p["about"] == f"https://the.link.example.com/what-was-linked-to {i}"

    def testOtherPrefix(self):
        xml_str = """
            <message to="whoever@example.com">
              <body>I wanted to mention https://the.link.example.com/what-was-linked-to</body>
              <ns1:Description xmlns:ns1="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                               xmlns:ns2="https://ogp.me/ns#"
                               ns1:about="https://the.link.example.com/what-was-linked-to">
                <ns2:title>Page Title</ns2:title>
                <ns2:description>Page Description</ns2:description>
                <ns2:url>Canonical URL</ns2:url>
                <ns2:type>website</ns2:type>
                <ns2:site_name>Some Website</ns2:site_name>
                <ns2:image>data:image/jpeg,...</ns2:image>
              </ns1:Description>
            </message>
            """
        xml = self.parse_xml(xml_str)
        msg = self.Message(xml)
        assert msg["link_metadata"]["title"] == "Page Title"
        assert msg["link_metadata"]["description"] == "Page Description"
        assert msg["link_metadata"]["url"] == "Canonical URL"
        assert msg["link_metadata"]["site_name"] == "Some Website"
        assert msg["link_metadata"]["type"] == "website"
        assert msg["link_metadata"]["image"] == "data:image/jpeg,..."
        assert (
            msg["link_metadata"]["about"]
            == "https://the.link.example.com/what-was-linked-to"
        )

suite = unittest.TestLoader().loadTestsFromTestCase(TestLinkMetadata)



from asyncio import Future

from slixmpp import Iq
from slixmpp.plugins import BasePlugin
from slixmpp.types import JidStr

from . import stanza


class XEP_0490(BasePlugin):
    """
    XEP-0490: Message Displayed Synchronization
    """

    name = "xep_0490"
    description = "XEP-0490: Message Displayed Synchronization"
    dependencies = {"xep_0060", "xep_0163", "xep_0223", "xep_0359"}
    stanza = stanza

    def plugin_init(self):
        stanza.register_plugin()
        self.xmpp.plugin["xep_0163"].register_pep(
            "message_displayed_synchronization",
            stanza.Displayed,
        )
        self.xmpp.plugin["xep_0223"].node_profiles[self.stanza.NS] = {
            "pubsub#max_items": "max",
            "pubsub#send_last_published_item": "never",
        }

    def flag_chat(self, chat: JidStr, stanza_id: str, **kwargs) -> Future[Iq]:
        """
        Flag a chat as displayed.

        :param chat: JID of the chat to set as displayed.
        :param stanza_id: stanza-id of which to set the display marker.

        """
        displayed = stanza.Displayed()
        displayed["stanza_id"]["id"] = stanza_id
        return self.xmpp.plugin["xep_0223"].store(
            displayed, node=stanza.NS, id=str(chat), **kwargs
        )

    def catch_up(self, **kwargs) -> Future[Iq]:
        """
        Get all displayed status.

        Can take any keyword parameters from XEP-0060’s ``get_items()``.
        """
        return self.xmpp.plugin["xep_0060"].get_items(
            self.xmpp.boundjid.bare, stanza.NS, **kwargs
        )



#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2015 Emmanuel Gil Peyrot
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import sys

import logging
from getpass import getpass
from argparse import ArgumentParser

import asyncio
import slixmpp
from slixmpp.exceptions import XMPPError, IqError

log = logging.getLogger(__name__)


class AskConfirm(slixmpp.ClientXMPP):

    """
    A basic client asking an entity if they confirm the access to an HTTP URL.
    """

    def __init__(self, jid, password, recipient, id, url, method):
        slixmpp.ClientXMPP.__init__(self, jid, password)

        self.recipient = recipient
        self.id = id
        self.url = url
        self.method = method

        # Will be used to set the proper exit code.
        self.confirmed = asyncio.Future()

        self.add_event_handler("session_start", self.start)
        self.add_event_handler("message", self.start)
        self.add_event_handler("http_confirm_message", self.confirm)

    def confirm(self, message):
        print(message)
        if message['confirm']['id'] == self.id:
            if message['type'] == 'error':
                self.confirmed.set_result(False)
            else:
                self.confirmed.set_result(True)

    async def start(self, event):
        log.info('Sending confirm request %s to %s who wants to access %s using '
                 'method %s...' % (self.id, self.recipient, self.url, self.method))
        try:
            confirmed = await self['xep_0070'].ask_confirm(self.recipient,
                                                                id=self.id,
                                                                url=self.url,
                                                                method=self.method,
                                                                message='Plz say yes or no for {method} {url} ({id}).')
            if isinstance(confirmed, slixmpp.Message):
                confirmed = await self.confirmed
            else:
                confirmed = True
        except IqError:
            confirmed = False
        if confirmed:
            print('Confirmed')
        else:
            print('Denied')
        self.disconnect()


if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser()
    parser.add_argument("-q","--quiet", help="set logging to ERROR",
                        action="store_const",
                        dest="loglevel",
                        const=logging.ERROR,
                        default=logging.INFO)
    parser.add_argument("-d","--debug", help="set logging to DEBUG",
                        action="store_const",
                        dest="loglevel",
                        const=logging.DEBUG,
                        default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")

    # Other options.
    parser.add_argument("-r", "--recipient", required=True,
                        help="Recipient JID")
    parser.add_argument("-i", "--id", required=True,
                        help="id TODO")
    parser.add_argument("-u", "--url", required=True,
                        help="URL the user tried to access")
    parser.add_argument("-m", "--method", required=True,
                        help="HTTP method used")

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = input("Username: ")
    if args.password is None:
        args.password = getpass("Password: ")

    xmpp = AskConfirm(args.jid, args.password, args.recipient, args.id,
                      args.url, args.method)
    xmpp.register_plugin('xep_0070')

    # Connect to the XMPP server and start processing XMPP stanzas.
    xmpp.connect()
    asyncio.get_event_loop().run_until_complete(xmpp.disconnected)
    sys.exit(0 if xmpp.confirmed else 1)



#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2026 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import logging
from getpass import getpass
from argparse import ArgumentParser
import sys

import datetime
import asyncio
import slixmpp
from slixmpp import JID

log = logging.getLogger(__name__)


async def get_stdin_reader():
    loop = asyncio.get_event_loop()
    reader = asyncio.StreamReader()
    protocol = asyncio.StreamReaderProtocol(reader)
    await loop.connect_read_pipe(lambda: protocol, sys.stdin)
    return reader


def parse_stdin(_in: bytes) -> list[tuple[str, list[int]]]:
    """
    Parse a string from stdin. Overcomplicated for what the script currently
    does.
    Spits out a list of (command, list of messages to act on).
    """
    stdin_str = _in.decode('utf-8')
    split_cmds = stdin_str.split(';')
    parsed_cmds = []
    for cmd in split_cmds:
        cmd = cmd.strip()
        split_cmd = cmd.split()
        if not split_cmd:
            continue
        cmd_verb = split_cmd.pop(0).lower()
        if cmd_verb not in ('d', 'q'):
            print(f"unknown command: {cmd}")
            continue
        if cmd_verb == 'q':
            break
        params = []
        for num in split_cmd:
            if '-' in num:
                num_split = num.split('-')
                if len(num_split) != 2:
                    print(f'Bad range: {num}')
                    continue
                try:
                    num1 = int(num_split[0])
                    num2 = int(num_split[1])
                    if num1 >= num2 or num1 < 0:
                        print(f'Bad range: {num}')
                        continue
                    params.extend(list(range(num1, num2+1)))
                except ValueError:
                    print(f'Bad range: {num}')
            else:
                try:
                    num_int = int(num)
                    if num_int < 0:
                        print(f'Bad number: {num}')
                    params.append(num_int)
                except ValueError:
                    print(f'Bad number: {num}')
        if not params:
            print(f'Bad command: {cmd}')
            continue
        parsed_cmds.append((cmd_verb, params))

    return parsed_cmds


class RoomCleanup(slixmpp.ClientXMPP):

    """
    A basic client fetching mam archive messages
    """

    def __init__(self, jid, password, remote_jid, start, max_messages):
        slixmpp.ClientXMPP.__init__(self, jid, password)
        self.remote_jid = JID(remote_jid)
        self.start_date = start
        self.messages = dict()
        self.max_messages = max_messages

        self.add_event_handler("session_start", self.start)

    async def start(self, *args):
        """
        Fetch mam results for the specified JID.
        Use RSM to paginate the results.
        """
        results = self.plugin['xep_0313'].retrieve(
            jid=self.remote_jid, iterator=True,
            rsm={'max': min(50, self.max_messages)},
            reverse=True, start=self.start_date,
        )
        number = 1
        body_msgs = 0
        stop = False
        msgs = []
        async for rsm in results:
            tmp_msgs = []
            for msg in rsm['mam']['results']:
                if msg['mam_result']['forwarded']['stanza']['body']:
                    body_msgs += 1
                    tmp_msgs.append(msg)
                if body_msgs >= self.max_messages:
                    stop = True
                    break
            msgs.extend(tmp_msgs[::-1])
            if stop:
                break
        for msg in msgs[::-1]:
            forwarded = msg['mam_result']['forwarded']
            timestamp = forwarded['delay']['stamp']
            message = forwarded['stanza']
            self.messages[number] = msg['mam_result']['id']
            header = f'{number: <6} [{timestamp}] {message["from"].resource}: '
            if not message['body']:
                continue
            if '\n' in message['body']:
                pad = ' ' * len(header)
                split = message['body'].split('\n')
                first = header + split.pop(0)
                lines = [first] + [pad + line for line in split]
            else:
                lines = [
                    f'{number: <6} [{timestamp}] '
                    f'{message["from"].resource}: {message["body"]}'
                ]
            for line in lines:
                print(line)
            number += 1
        print(
            'Enter a command ("d" for delete or "q" for quit) '
            'followed by message numbers or ranges:\n'
            'example: d 1-5 17 53'
        )
        reader = await get_stdin_reader()
        commands = parse_stdin(await reader.read(1024))
        for command, params in commands:
            match command:
                case "d":
                    await self.delete_messages(params)
                case "q":
                    break

        await self.disconnect()

    async def delete_messages(self, numbers: list[int]) -> None:
        """Actually delete the messages with the IDs kept in cache"""
        success = 0
        for number in numbers:
            if number not in self.messages:
                print(f'Unknown message number: {number}')
                continue
            msg_id = self.messages[number]
            try:
                await self.plugin['xep_0425'].moderate(
                    self.remote_jid,
                    msg_id,
                    reason='moderated',
                )
                success += 1
            except Exception as exc:
                print(f'Failed to moderate message: {exc}')
        print(f'{success} message(s) successfully deleted.')


if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser(
        description=(
            "Script to clean up the room using message moderation. \n"
            "Messages will be listed with a prepended number, and can be then "
            "moderated by using the 'd' command, either individually or "
            "with a range.\n"
        ),
        epilog=(
            "You must have moderating powers in the room "
            "for the script to work."
        ),
    )
    parser.add_argument("-q", "--quiet", help="set logging to ERROR",
                        action="store_const",
                        dest="loglevel",
                        const=logging.ERROR,
                        default=logging.INFO)
    parser.add_argument("-d", "--debug", help="set logging to DEBUG",
                        action="store_const",
                        dest="loglevel",
                        const=logging.DEBUG,
                        default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")

    # Other options
    parser.add_argument("-r", "--remote-jid", dest="remote_jid",
                        help="Remote JID")
    today = datetime.datetime.now().strftime('%Y-%m-%dT00:00:00Z')
    parser.add_argument("--start", help="Start date", default=today)
    parser.add_argument("-m", "--messages", help="Number of messages to fetch",
                        type=int, default=50)

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = input("Username: ")
    if args.password is None:
        args.password = getpass("Password: ")
    if args.remote_jid is None:
        args.remote_jid = input("Remote JID: ")
    if args.start is None:
        args.start = input("Start time: ")

    xmpp = RoomCleanup(args.jid, args.password, args.remote_jid,
                       args.start, args.messages)
    xmpp.register_plugin('xep_0313')
    xmpp.register_plugin('xep_0425')

    # Connect to the XMPP server and start processing XMPP stanzas.
    xmpp.connect()
    asyncio.get_event_loop().run_until_complete(xmpp.disconnected)



# -*- encoding:utf-8 -*-

"""
    Slixmpp: The Slick XMPP Library
    Copyright (C) 2011 Nathanael C. Fritz, Dann Martens (TOMOTON).
    This file is part of Slixmpp.

    See the file LICENSE for copying permission.
"""

import base64
import sys

from slixmpp.plugins.xep_0009.stanza import RPCQuery, MethodCall, \
    MethodResponse
from slixmpp.plugins.xep_0009.binding import py2xml, xml2py, rpcbase64, \
    rpctime
from slixmpp.stanza.iq import Iq
from slixmpp.test.slixtest import SlixTest
from slixmpp.xmlstream.stanzabase import register_stanza_plugin
from slixmpp.xmlstream.tostring import tostring
import unittest


class TestJabberRPC(SlixTest):

    def setUp(self):
        register_stanza_plugin(Iq, RPCQuery)
        register_stanza_plugin(RPCQuery, MethodCall)
        register_stanza_plugin(RPCQuery, MethodResponse)

    def testMethodCall(self):
        iq = self.Iq()
        iq['rpc_query']['method_call']['method_name'] = 'system.exit'
        iq['rpc_query']['method_call']['params'] = py2xml(*())
        self.check(iq, """
            <iq>
                <query xmlns="jabber:iq:rpc">
                    <methodCall>
                        <methodName>system.exit</methodName>
                        <params />
                    </methodCall>
                </query>
            </iq>
        """, use_values=False)

    def testMethodResponse(self):
        iq = self.Iq()
        iq['rpc_query']['method_response']['params'] = py2xml(*())
        self.check(iq, """
            <iq>
                <query xmlns="jabber:iq:rpc">
                    <methodResponse>
                        <params />
                    </methodResponse>
                </query>
            </iq>
        """, use_values=False)

    def testConvertNil(self):
        params = [None]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <nil />
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Nil to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                         "XML to nil conversion")

    def testConvertBoolean(self):
        params = [True, False]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <boolean>1</boolean>
                    </value>
                </param>
                <param>
                    <value>
                        <boolean>0</boolean>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Boolean to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                         "XML to boolean conversion")

    def testConvertString(self):
        params = ["'This' & \"That\""]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <string>&apos;This&apos; &amp; &quot;That&quot;</string>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "String to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                        "XML to string conversion")

    def testConvertUnicodeString(self):
        params = ["おはよう"]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <string>おはよう</string>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "String to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                        "XML to string conversion")

    def testConvertInteger(self):
        params = [32767, -32768]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <i4>32767</i4>
                    </value>
                </param>
                <param>
                    <value>
                        <i4>-32768</i4>
                    </value>
                </param>
            </params>
        """)
        alternate_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <int>32767</int>
                    </value>
                </param>
                <param>
                    <value>
                        <int>-32768</int>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Integer to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                         "XML to boolean conversion")
        self.assertEqual(params, xml2py(alternate_xml),
                         "Alternate XML to boolean conversion")


    def testConvertDouble(self):
        params = [3.14159265]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <double>3.14159265</double>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Double to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                         "XML to double conversion")

    def testConvertBase64(self):
        params = [rpcbase64(base64.b64encode(b"Hello, world!"))]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <base64>SGVsbG8sIHdvcmxkIQ==</base64>
                    </value>
                </param>
            </params>
        """)
        alternate_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <Base64>SGVsbG8sIHdvcmxkIQ==</Base64>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Base64 to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(list(map(lambda x: x.decode(), params)),
                         list(map(lambda x: x.decode(), xml2py(expected_xml))),
                         "XML to base64 conversion")
        self.assertEqual(list(map(lambda x: x.decode(), params)),
                         list(map(lambda x: x.decode(), xml2py(alternate_xml))),
                         "Alternate XML to base64 conversion")

    def testConvertDateTime(self):
        params = [rpctime("20111220T01:50:00")]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <dateTime.iso8601>20111220T01:50:00</dateTime.iso8601>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "DateTime to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(list(map(lambda x: x.iso8601(), params)),
                         list(map(lambda x: x.iso8601(), xml2py(expected_xml))),
                         None)

    def testConvertArray(self):
        params = [[1,2,3], ('a', 'b', 'c')]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <array>
                            <data>
                                <value><i4>1</i4></value>
                                <value><i4>2</i4></value>
                                <value><i4>3</i4></value>
                            </data>
                        </array>
                    </value>
                </param>
                <param>
                    <value>
                        <array>
                            <data>
                                <value><string>a</string></value>
                                <value><string>b</string></value>
                                <value><string>c</string></value>
                            </data>
                        </array>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Array to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(list(map(list, params)), xml2py(expected_xml),
                         "XML to array conversion")

    def testConvertStruct(self):
        params = [{"foo": "bar", "baz": False}]
        params_xml = py2xml(*params)
        expected_xml = self.parse_xml("""
            <params xmlns="jabber:iq:rpc">
                <param>
                    <value>
                        <struct>
                            <member>
                                <name>foo</name>
                                <value><string>bar</string></value>
                            </member>
                            <member>
                                <name>baz</name>
                                <value><boolean>0</boolean></value>
                            </member>
                        </struct>
                    </value>
                </param>
            </params>
        """)
        self.assertTrue(self.compare(expected_xml, params_xml),
                        "Struct to XML conversion\nExpected: %s\nGot: %s" % (
                            tostring(expected_xml), tostring(params_xml)))
        self.assertEqual(params, xml2py(expected_xml),
                         "XML to struct conversion")

suite = unittest.TestLoader().loadTestsFromTestCase(TestJabberRPC)




import unittest
from slixmpp.test.integration import SlixIntegration

try:
    import aiohttp
except ImportError:
    aiohttp = None


class TestSos(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        self.register_plugins(['xep_0455'])
        await self.connect_clients()

    @unittest.skipIf(aiohttp is None, "aiohttp is not installed")
    async def test_sos(self):
        """Check we can get the status addr and fetch empty data"""
        addresses = await self.clients[0]['xep_0455'].get_external_status_addresses()
        self.assertGreaterEqual(len(addresses), 1)
        fetched_status = await self.clients[0]['xep_0455'].fetch_status(addresses[0])
        self.assertEqual(fetched_status, None)


suite = unittest.TestLoader().loadTestsFromTestCase(TestSos)



import unittest
import unittest.mock

from slixmpp.test import SlixTest


class TestMUC(SlixTest):
    def setUp(self):
        self.stream_start(plugins=['xep_0045', 'xep_0030'])

    def test_new_affiliation(self):
        handler = unittest.mock.AsyncMock()
        self.xmpp.add_event_handler("muc::room@rooms.test::affiliation_change", handler)
        self.recv(  # language=XML
            """
            <message from="room@rooms.test">
              <x xmlns="http://jabber.org/protocol/muc#user">
                <item jid="caca@prout"
                      affiliation="member" />
              </x>
            </message>
            """
        )
        handler.assert_awaited_once()
        msg = handler.call_args.args[0]
        assert msg["muc"]["item"]["jid"] == "caca@prout"
        assert msg["muc"]["item"]["affiliation"] == "member"

    def test_remove_affiliation(self):
        handler = unittest.mock.AsyncMock()
        self.xmpp.add_event_handler("muc::room@rooms.test::affiliation_change", handler)
        self.recv(  # language=XML
            """
            <message from="room@rooms.test">
              <x xmlns="http://jabber.org/protocol/muc#user">
                <status code="321" />
                <item jid="caca@prout"
                      affiliation="none" />
              </x>
            </message>
            """
        )
        handler.assert_awaited_once()
        msg = handler.call_args.args[0]
        assert msg["muc"]["item"]["jid"] == "caca@prout"
        assert msg["muc"]["item"]["affiliation"] == "none"


suite = unittest.TestLoader().loadTestsFromTestCase(TestMUC)



#!/usr/bin/env python

"""
A small script to update the DOAP file and commit/tag it.

Usage: ./release.py 2.0.0
"""

import fileinput
import sys
from datetime import datetime
from subprocess import check_call

version = sys.argv[1]

doap_release = f"""    <release>
        <Version>
            <revision>{version}</revision>
            <created>{datetime.now():%Y-%m-%d}</created>
            <file-release rdf:resource="https://codeberg.org/poezio/slixmpp/archive/slix-{version}.tar.gz"/>
        </Version>
    </release>
</Project>"""

try:
    proud, default, shame = (int(x) for x in version.split("."))
except ValueError:
    print("Not a valid version")
    exit(1)

for line in fileinput.input(files=["doap.xml"], inplace=True):
    if line.startswith("</Project>"):
        print(doap_release)
    else:
        print(line, end="")

check_call(["git", "commit", "doap.xml", "-m", f"build: release {version}"])
check_call(["git", "tag", "-s", f"slix-{version}"])




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2012 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import logging
from asyncio import Future

from slixmpp import JID
from slixmpp.stanza import Iq
from slixmpp.plugins import BasePlugin
from slixmpp.xmlstream.handler import Callback
from slixmpp.xmlstream.matcher import StanzaPath
from slixmpp.xmlstream import register_stanza_plugin, ElementBase
from slixmpp.plugins.xep_0049 import stanza, PrivateXML


log = logging.getLogger(__name__)


class XEP_0049(BasePlugin):

    name = 'xep_0049'
    description = 'XEP-0049: Private XML Storage'
    dependencies = {}
    stanza = stanza

    def plugin_init(self):
        register_stanza_plugin(Iq, PrivateXML)

    def register(self, stanza):
        register_stanza_plugin(PrivateXML, stanza, iterable=True)

    def store(self, data: list[ElementBase] | ElementBase, ifrom: JID | None = None, **iqkwargs) -> Future:
        """Store data in Private XML Storage.

        :param data: An XML element or list of xml element to store.
        """
        iq = self.xmpp.make_iq_set(ifrom=ifrom)

        if not isinstance(data, list):
            data = [data]
        for elem in data:
            iq['private'].append(elem)

        return iq.send(**iqkwargs)

    def retrieve(self, name: str, ifrom: JID | None = None, **iqkwargs) -> Future:
        """Get previously stored data from Private XML Storage.

        :param name: Name of the payload to retrieve (slixmpp plugin attribute)
        """
        iq = self.xmpp.make_iq_get(ifrom=ifrom)
        iq['private'].enable(name)
        return iq.send(**iqkwargs)
